# LICENSE HEADER MANAGED BY add-license-header
#
# SPDX-FileCopyrightText: Copyright 2024 German Cancer Research Center (DKFZ) and contributors.
# SPDX-License-Identifier: MIT
#

from pathlib import Path

from mml.core.data_loading.task_attributes import Keyword, License, Modality, TaskType
from mml.core.data_preparation.data_archive import DataKind
from mml.core.data_preparation.dset_creator import DSetCreator
from mml.core.data_preparation.registry import register_dsetcreator, register_taskcreator
from mml.core.data_preparation.task_creator import TaskCreator

REFERENCE = """"
@inproceedings{DBLP:conf/mmm/LeibetsederKSKK20,
    author    = {Andreas Leibetseder and
                    Sabrina Kletz and
                    Klaus Schoeffmann and
                    Simon Keckstein and
                    J{\"{o}}rg Keckstein},
    title     = {{GLENDA:} Gynecologic Laparoscopy Endometriosis Dataset},
    booktitle = {MultiMedia Modeling - 26th International Conference, {MMM} 2020, Daejeon,
                    South Korea, January 5-8, 2020, Proceedings, Part {II}},
    series    = {Lecture Notes in Computer Science},
    volume    = {11962},
    pages     = {439--450},
    publisher = {Springer},
    year      = {2020},
    url       = {https://doi.org/10.1007/978-3-030-37734-2\\_36},
    doi       = {10.1007/978-3-030-37734-2\\_36},
    timestamp = {Mon, 09 Nov 2020 15:46:42 +0100},
    biburl    = {https://dblp.org/rec/conf/mmm/LeibetsederKSKK20.bib},
    bibsource = {dblp computer science bibliography, https://dblp.org}
}
"""

dset_name = "glenda"
class_mapping = {
    (0, 0, 0): "background",  # Background/No Pathology
    (190, 62, 204): "6.1.1.1_Endo-Peritoneum",  # Peritoneal endometriosis
    (93, 89, 254): "6.1.1.2_Endo-Ovar",  # Endometriosis on ovaries
    (145, 210, 138): "6.1.1.3_Endo-TIE",
    # Deep infiltrating endometriosis( found on specific locations such as the rectum, the rectovaginal space
    # or uterine ligaments)
    (238, 236, 50): "6.1.1.4_Endo-Uterus",  # Uterine endometriosis or Adenomyosis
}
# these values are borderline pixels, not directly mappable to one of the classes, they will be ignored
wrongs = [
    [0, 0, 1],
    [1, 0, 1],
    [1, 0, 2],
    [1, 1, 0],
    [1, 1, 1],
    [1, 1, 2],
    [1, 1, 3],
    [1, 1, 4],
    [1, 2, 1],
    [2, 1, 2],
    [2, 2, 0],
    [2, 2, 2],
    [2, 2, 5],
    [2, 2, 6],
    [2, 3, 2],
    [3, 1, 3],
    [3, 2, 7],
    [3, 3, 1],
    [3, 3, 8],
    [3, 3, 9],
    [3, 4, 3],
    [3, 5, 3],
    [4, 1, 4],
    [4, 1, 5],
    [4, 3, 10],
    [4, 4, 1],
    [4, 4, 11],
    [4, 4, 12],
    [4, 6, 4],
    [5, 2, 6],
    [5, 5, 1],
    [5, 5, 13],
    [5, 5, 14],
    [5, 5, 15],
    [5, 7, 4],
    [5, 7, 5],
    [6, 2, 6],
    [6, 6, 1],
    [6, 6, 16],
    [6, 6, 17],
    [6, 8, 5],
    [6, 9, 6],
    [7, 2, 7],
    [7, 2, 8],
    [7, 6, 1],
    [7, 6, 18],
    [7, 7, 2],
    [7, 7, 19],
    [7, 7, 20],
    [7, 10, 6],
    [7, 11, 7],
    [8, 3, 9],
    [8, 7, 21],
    [8, 8, 2],
    [8, 8, 22],
    [8, 8, 23],
    [8, 11, 8],
    [8, 12, 8],
    [9, 3, 10],
    [9, 8, 24],
    [9, 9, 2],
    [9, 9, 25],
    [9, 9, 26],
    [9, 13, 9],
    [10, 3, 10],
    [10, 3, 11],
    [10, 9, 27],
    [10, 10, 2],
    [10, 10, 28],
    [10, 14, 9],
    [10, 15, 10],
    [11, 4, 12],
    [11, 10, 29],
    [11, 10, 30],
    [11, 11, 2],
    [11, 11, 31],
    [11, 16, 10],
    [11, 16, 11],
    [12, 4, 13],
    [12, 11, 32],
    [12, 11, 33],
    [12, 12, 3],
    [12, 12, 34],
    [12, 17, 11],
    [12, 18, 12],
    [13, 4, 14],
    [13, 12, 35],
    [13, 13, 3],
    [13, 13, 36],
    [13, 13, 37],
    [13, 19, 12],
    [14, 5, 15],
    [14, 13, 38],
    [14, 14, 3],
    [14, 14, 39],
    [14, 20, 13],
    [14, 21, 13],
    [15, 5, 16],
    [15, 14, 40],
    [15, 14, 41],
    [15, 15, 3],
    [15, 15, 42],
    [15, 21, 14],
    [15, 22, 15],
    [16, 5, 17],
    [16, 5, 18],
    [16, 15, 43],
    [16, 15, 44],
    [16, 16, 3],
    [16, 16, 45],
    [16, 23, 15],
    [16, 24, 16],
    [17, 6, 18],
    [17, 16, 46],
    [17, 16, 47],
    [17, 17, 4],
    [17, 17, 48],
    [17, 25, 16],
    [18, 6, 19],
    [18, 17, 49],
    [18, 17, 50],
    [18, 18, 4],
    [18, 25, 17],
    [18, 26, 17],
    [19, 6, 20],
    [19, 6, 21],
    [19, 18, 4],
    [19, 18, 51],
    [19, 18, 52],
    [19, 18, 53],
    [19, 27, 18],
    [19, 28, 18],
    [20, 7, 22],
    [20, 19, 4],
    [20, 19, 54],
    [20, 19, 55],
    [20, 19, 56],
    [20, 20, 4],
    [20, 29, 19],
    [20, 30, 19],
    [21, 7, 22],
    [21, 20, 57],
    [21, 20, 58],
    [21, 21, 4],
    [21, 21, 59],
    [21, 30, 20],
    [22, 7, 23],
    [22, 7, 24],
    [22, 21, 60],
    [22, 21, 61],
    [22, 22, 5],
    [22, 31, 20],
    [22, 32, 21],
    [23, 8, 25],
    [23, 22, 62],
    [23, 22, 63],
    [23, 22, 64],
    [23, 23, 5],
    [23, 33, 22],
    [23, 34, 22],
    [24, 8, 26],
    [24, 23, 64],
    [24, 23, 65],
    [24, 23, 66],
    [24, 24, 5],
    [24, 34, 23],
    [24, 35, 23],
    [25, 8, 27],
    [25, 24, 67],
    [25, 24, 68],
    [25, 24, 69],
    [25, 25, 5],
    [25, 36, 24],
    [25, 37, 24],
    [26, 8, 28],
    [26, 25, 70],
    [26, 25, 71],
    [26, 26, 5],
    [26, 38, 25],
    [27, 9, 29],
    [27, 25, 72],
    [27, 26, 73],
    [27, 26, 74],
    [27, 27, 6],
    [27, 39, 25],
    [27, 39, 26],
    [28, 9, 30],
    [28, 26, 75],
    [28, 27, 76],
    [28, 27, 77],
    [28, 28, 6],
    [28, 40, 26],
    [28, 41, 27],
    [29, 9, 31],
    [29, 9, 32],
    [29, 27, 78],
    [29, 28, 79],
    [29, 28, 80],
    [29, 29, 6],
    [29, 42, 27],
    [29, 43, 28],
    [30, 10, 32],
    [30, 10, 33],
    [30, 29, 81],
    [30, 29, 82],
    [30, 30, 6],
    [30, 43, 29],
    [31, 10, 33],
    [31, 29, 83],
    [31, 30, 6],
    [31, 30, 84],
    [31, 30, 85],
    [31, 44, 29],
    [31, 45, 30],
    [32, 10, 34],
    [32, 30, 86],
    [32, 31, 7],
    [32, 31, 87],
    [32, 31, 88],
    [32, 46, 30],
    [32, 47, 31],
    [33, 11, 35],
    [33, 11, 36],
    [33, 31, 89],
    [33, 32, 7],
    [33, 32, 90],
    [33, 32, 91],
    [33, 33, 7],
    [33, 48, 31],
    [33, 48, 32],
    [34, 11, 37],
    [34, 32, 92],
    [34, 33, 93],
    [34, 34, 7],
    [34, 49, 32],
    [35, 11, 37],
    [35, 33, 94],
    [35, 33, 95],
    [35, 34, 96],
    [35, 35, 7],
    [35, 50, 33],
    [35, 51, 33],
    [36, 12, 38],
    [36, 12, 39],
    [36, 34, 97],
    [36, 34, 98],
    [36, 35, 99],
    [36, 36, 8],
    [36, 52, 34],
    [36, 53, 35],
    [37, 12, 40],
    [37, 35, 100],
    [37, 35, 101],
    [37, 36, 102],
    [37, 37, 8],
    [37, 53, 35],
    [37, 54, 36],
    [38, 12, 41],
    [38, 36, 103],
    [38, 37, 104],
    [38, 38, 8],
    [38, 55, 36],
    [39, 13, 41],
    [39, 13, 42],
    [39, 37, 105],
    [39, 37, 106],
    [39, 38, 107],
    [39, 39, 8],
    [39, 56, 37],
    [39, 57, 37],
    [40, 13, 43],
    [40, 38, 108],
    [40, 38, 109],
    [40, 39, 110],
    [40, 40, 8],
    [40, 57, 38],
    [40, 58, 38],
    [41, 13, 44],
    [41, 39, 111],
    [41, 39, 112],
    [41, 40, 113],
    [41, 41, 9],
    [41, 59, 39],
    [41, 60, 39],
    [42, 14, 45],
    [42, 40, 114],
    [42, 40, 115],
    [42, 41, 9],
    [42, 61, 40],
    [42, 62, 40],
    [43, 14, 46],
    [43, 41, 116],
    [43, 41, 117],
    [43, 41, 118],
    [43, 42, 9],
    [43, 62, 41],
    [44, 14, 47],
    [44, 42, 119],
    [44, 42, 120],
    [44, 42, 121],
    [44, 43, 9],
    [44, 63, 42],
    [44, 64, 42],
    [45, 15, 48],
    [45, 15, 49],
    [45, 43, 122],
    [45, 43, 123],
    [45, 43, 124],
    [45, 44, 9],
    [45, 65, 43],
    [45, 66, 43],
    [46, 15, 49],
    [46, 44, 125],
    [46, 44, 126],
    [46, 45, 10],
    [46, 46, 10],
    [46, 66, 44],
    [46, 67, 44],
    [47, 15, 50],
    [47, 45, 128],
    [47, 45, 129],
    [47, 47, 10],
    [47, 68, 45],
    [48, 16, 51],
    [48, 16, 52],
    [48, 46, 130],
    [48, 46, 131],
    [48, 46, 132],
    [48, 48, 10],
    [48, 69, 45],
    [48, 70, 46],
    [49, 16, 53],
    [49, 47, 133],
    [49, 47, 134],
    [49, 47, 135],
    [49, 49, 10],
    [49, 71, 46],
    [49, 71, 47],
    [50, 16, 53],
    [50, 16, 54],
    [50, 48, 136],
    [50, 48, 137],
    [50, 48, 138],
    [50, 50, 11],
    [50, 72, 47],
    [50, 73, 48],
    [51, 17, 55],
    [51, 49, 139],
    [51, 49, 140],
    [51, 51, 11],
    [51, 74, 49],
    [52, 17, 56],
    [52, 49, 141],
    [52, 50, 142],
    [52, 50, 143],
    [52, 52, 11],
    [52, 75, 49],
    [52, 75, 50],
    [53, 17, 57],
    [53, 50, 144],
    [53, 51, 145],
    [53, 51, 146],
    [53, 53, 11],
    [53, 76, 50],
    [53, 77, 50],
    [53, 77, 51],
    [54, 18, 58],
    [54, 51, 147],
    [54, 52, 148],
    [54, 52, 149],
    [54, 53, 11],
    [54, 78, 51],
    [54, 79, 52],
    [55, 18, 59],
    [55, 52, 150],
    [55, 53, 151],
    [55, 54, 12],
    [55, 80, 52],
    [56, 18, 60],
    [56, 18, 61],
    [56, 53, 152],
    [56, 54, 153],
    [56, 54, 154],
    [56, 55, 12],
    [56, 80, 53],
    [56, 81, 53],
    [57, 19, 61],
    [57, 54, 155],
    [57, 55, 156],
    [57, 55, 157],
    [57, 56, 12],
    [57, 82, 54],
    [57, 83, 54],
    [58, 19, 62],
    [58, 55, 158],
    [58, 56, 159],
    [58, 56, 160],
    [58, 57, 12],
    [58, 84, 55],
    [58, 84, 56],
    [59, 19, 63],
    [59, 19, 64],
    [59, 56, 161],
    [59, 57, 162],
    [59, 58, 12],
    [59, 85, 56],
    [59, 86, 57],
    [60, 20, 65],
    [60, 57, 163],
    [60, 57, 164],
    [60, 58, 165],
    [60, 59, 13],
    [60, 60, 13],
    [60, 87, 57],
    [61, 20, 65],
    [61, 58, 166],
    [61, 58, 167],
    [61, 59, 168],
    [61, 61, 13],
    [61, 88, 58],
    [61, 89, 58],
    [62, 20, 66],
    [62, 20, 67],
    [62, 59, 169],
    [62, 59, 170],
    [62, 60, 171],
    [62, 62, 13],
    [62, 89, 58],
    [62, 89, 59],
    [62, 90, 59],
    [63, 21, 68],
    [63, 60, 172],
    [63, 60, 173],
    [63, 63, 13],
    [63, 91, 60],
    [63, 92, 60],
    [64, 21, 69],
    [64, 61, 174],
    [64, 61, 175],
    [64, 62, 176],
    [64, 64, 13],
    [64, 93, 61],
    [65, 21, 69],
    [65, 21, 70],
    [65, 62, 177],
    [65, 62, 178],
    [65, 63, 179],
    [65, 65, 14],
    [65, 94, 61],
    [65, 94, 62],
    [66, 22, 71],
    [66, 63, 180],
    [66, 63, 181],
    [66, 64, 182],
    [66, 65, 14],
    [66, 95, 63],
    [66, 96, 63],
    [67, 22, 71],
    [67, 22, 72],
    [67, 64, 183],
    [67, 64, 184],
    [67, 66, 14],
    [67, 97, 64],
    [67, 98, 64],
    [68, 22, 73],
    [68, 65, 185],
    [68, 65, 186],
    [68, 65, 187],
    [68, 67, 14],
    [68, 98, 65],
    [69, 23, 74],
    [69, 66, 188],
    [69, 66, 189],
    [69, 66, 190],
    [69, 68, 14],
    [69, 99, 65],
    [69, 100, 66],
    [70, 23, 75],
    [70, 67, 191],
    [70, 67, 192],
    [70, 69, 15],
    [70, 101, 66],
    [70, 102, 67],
    [71, 23, 76],
    [71, 23, 77],
    [71, 68, 193],
    [71, 68, 194],
    [71, 70, 15],
    [71, 103, 67],
    [71, 103, 68],
    [72, 23, 77],
    [72, 68, 195],
    [72, 69, 196],
    [72, 69, 197],
    [72, 71, 15],
    [72, 104, 68],
    [73, 24, 78],
    [73, 24, 79],
    [73, 70, 198],
    [73, 70, 199],
    [73, 70, 200],
    [73, 72, 15],
    [73, 73, 15],
    [73, 106, 70],
    [74, 24, 80],
    [74, 71, 201],
    [74, 71, 202],
    [74, 71, 203],
    [74, 74, 16],
    [74, 107, 70],
    [74, 107, 71],
    [75, 24, 80],
    [75, 72, 204],
    [75, 72, 205],
    [75, 75, 16],
    [75, 108, 71],
    [75, 109, 72],
    [76, 25, 81],
    [76, 25, 82],
    [76, 72, 206],
    [76, 73, 207],
    [76, 73, 208],
    [76, 76, 16],
    [76, 110, 72],
    [76, 111, 73],
    [77, 25, 83],
    [77, 73, 209],
    [77, 74, 210],
    [77, 74, 211],
    [77, 77, 16],
    [77, 112, 73],
    [78, 25, 84],
    [78, 74, 212],
    [78, 75, 213],
    [78, 75, 214],
    [78, 77, 16],
    [78, 112, 74],
    [78, 113, 74],
    [79, 26, 84],
    [79, 26, 85],
    [79, 75, 215],
    [79, 76, 216],
    [79, 78, 17],
    [79, 114, 75],
    [79, 115, 75],
    [80, 26, 86],
    [80, 76, 217],
    [80, 76, 218],
    [80, 77, 219],
    [80, 79, 17],
    [80, 116, 76],
    [80, 116, 77],
    [81, 26, 87],
    [81, 77, 220],
    [81, 78, 221],
    [81, 78, 222],
    [81, 80, 17],
    [81, 117, 77],
    [82, 27, 88],
    [82, 78, 223],
    [82, 79, 224],
    [82, 79, 225],
    [82, 81, 17],
    [82, 118, 78],
    [82, 119, 78],
    [83, 27, 89],
    [83, 79, 226],
    [83, 80, 227],
    [83, 82, 17],
    [83, 120, 79],
    [83, 121, 79],
    [84, 27, 90],
    [84, 80, 228],
    [84, 80, 229],
    [84, 81, 230],
    [84, 83, 18],
    [84, 121, 80],
    [84, 122, 80],
    [85, 28, 91],
    [85, 28, 92],
    [85, 81, 231],
    [85, 81, 232],
    [85, 82, 233],
    [85, 84, 18],
    [85, 123, 81],
    [86, 28, 92],
    [86, 82, 234],
    [86, 82, 235],
    [86, 83, 236],
    [86, 85, 18],
    [86, 86, 18],
    [86, 124, 81],
    [86, 125, 82],
    [87, 28, 93],
    [87, 29, 93],
    [87, 83, 236],
    [87, 83, 237],
    [87, 83, 238],
    [87, 84, 238],
    [87, 84, 239],
    [87, 87, 18],
    [87, 126, 82],
    [87, 126, 83],
    [88, 29, 94],
    [88, 29, 95],
    [88, 84, 239],
    [88, 84, 240],
    [88, 84, 241],
    [88, 85, 241],
    [88, 85, 242],
    [88, 88, 19],
    [88, 127, 84],
    [88, 128, 84],
    [89, 29, 96],
    [89, 85, 242],
    [89, 85, 243],
    [89, 85, 244],
    [89, 86, 244],
    [89, 89, 19],
    [89, 129, 85],
    [89, 130, 85],
    [90, 29, 96],
    [90, 86, 244],
    [90, 86, 245],
    [90, 86, 246],
    [90, 86, 247],
    [90, 87, 247],
    [90, 89, 19],
    [90, 130, 86],
    [91, 30, 97],
    [91, 30, 98],
    [91, 87, 247],
    [91, 87, 248],
    [91, 87, 249],
    [91, 90, 19],
    [91, 131, 86],
    [91, 132, 87],
    [92, 30, 99],
    [92, 88, 250],
    [92, 88, 251],
    [92, 88, 252],
    [92, 91, 19],
    [92, 133, 87],
    [92, 134, 88],
    [93, 30, 100],
    [93, 89, 253],
    [93, 92, 20],
    [93, 135, 88],
    [93, 135, 89],
    [94, 31, 100],
    [94, 31, 101],
    [94, 93, 20],
    [94, 136, 89],
    [95, 94, 20],
    [95, 137, 90],
    [95, 138, 91],
    [96, 31, 103],
    [96, 31, 104],
    [96, 95, 20],
    [96, 139, 91],
    [96, 139, 92],
    [97, 32, 104],
    [97, 96, 20],
    [97, 140, 92],
    [97, 141, 93],
    [98, 32, 105],
    [98, 97, 21],
    [98, 142, 93],
    [99, 32, 106],
    [99, 32, 107],
    [99, 98, 21],
    [99, 99, 21],
    [99, 143, 94],
    [99, 144, 94],
    [100, 33, 108],
    [100, 100, 21],
    [100, 144, 95],
    [100, 145, 95],
    [101, 33, 108],
    [101, 100, 21],
    [101, 146, 96],
    [101, 147, 96],
    [102, 33, 109],
    [102, 33, 110],
    [102, 101, 21],
    [102, 148, 97],
    [103, 34, 111],
    [103, 102, 22],
    [103, 148, 98],
    [103, 149, 98],
    [104, 34, 112],
    [104, 103, 22],
    [104, 150, 99],
    [104, 151, 99],
    [105, 34, 112],
    [105, 34, 113],
    [105, 104, 22],
    [105, 152, 100],
    [105, 153, 100],
    [106, 35, 114],
    [106, 105, 22],
    [106, 153, 101],
    [106, 154, 101],
    [107, 35, 115],
    [107, 106, 22],
    [107, 155, 102],
    [108, 35, 116],
    [108, 107, 23],
    [108, 156, 102],
    [108, 157, 103],
    [109, 36, 117],
    [109, 108, 23],
    [109, 158, 104],
    [110, 36, 118],
    [110, 109, 23],
    [110, 159, 105],
    [110, 160, 105],
    [111, 36, 119],
    [111, 36, 120],
    [111, 110, 23],
    [111, 161, 106],
    [112, 37, 120],
    [112, 111, 23],
    [112, 112, 24],
    [112, 162, 106],
    [112, 162, 107],
    [113, 37, 121],
    [113, 112, 24],
    [113, 163, 107],
    [113, 164, 108],
    [114, 37, 122],
    [114, 37, 123],
    [114, 113, 24],
    [114, 165, 108],
    [114, 166, 109],
    [115, 38, 124],
    [115, 114, 24],
    [115, 167, 109],
    [116, 38, 124],
    [116, 115, 24],
    [116, 167, 110],
    [116, 168, 111],
    [117, 38, 125],
    [117, 38, 126],
    [117, 116, 25],
    [117, 169, 111],
    [117, 170, 112],
    [118, 39, 127],
    [118, 117, 25],
    [118, 171, 112],
    [118, 171, 113],
    [119, 39, 128],
    [119, 172, 113],
    [120, 39, 129],
    [120, 40, 130],
    [120, 119, 25],
    [120, 173, 114],
    [120, 174, 114],
    [121, 39, 130],
    [121, 120, 25],
    [121, 175, 115],
    [121, 176, 115],
    [122, 40, 131],
    [122, 121, 26],
    [122, 176, 116],
    [122, 177, 116],
    [123, 40, 132],
    [123, 122, 26],
    [123, 178, 117],
    [123, 179, 118],
    [124, 40, 133],
    [124, 40, 134],
    [124, 123, 26],
    [124, 180, 118],
    [125, 41, 134],
    [125, 41, 135],
    [125, 124, 26],
    [125, 180, 119],
    [125, 181, 119],
    [126, 41, 135],
    [126, 124, 26],
    [126, 125, 27],
    [126, 182, 120],
    [126, 183, 120],
    [127, 41, 136],
    [127, 126, 27],
    [127, 184, 121],
    [127, 185, 121],
    [128, 42, 137],
    [128, 42, 138],
    [128, 127, 27],
    [128, 185, 121],
    [128, 185, 122],
    [128, 186, 122],
    [129, 42, 139],
    [129, 128, 27],
    [129, 186, 122],
    [129, 187, 123],
    [130, 42, 139],
    [130, 129, 27],
    [130, 188, 123],
    [130, 188, 124],
    [130, 189, 124],
    [131, 43, 140],
    [131, 43, 141],
    [131, 130, 28],
    [131, 189, 125],
    [131, 190, 125],
    [132, 43, 142],
    [132, 131, 28],
    [132, 190, 125],
    [132, 191, 125],
    [132, 191, 126],
    [133, 43, 143],
    [133, 132, 28],
    [133, 192, 126],
    [133, 193, 127],
    [134, 44, 143],
    [134, 44, 144],
    [134, 133, 28],
    [134, 194, 127],
    [134, 194, 128],
    [134, 195, 128],
    [135, 44, 145],
    [135, 134, 28],
    [135, 195, 128],
    [135, 196, 129],
    [136, 44, 146],
    [136, 135, 29],
    [136, 196, 129],
    [136, 197, 129],
    [136, 197, 130],
    [137, 44, 147],
    [137, 45, 147],
    [137, 136, 29],
    [137, 198, 130],
    [137, 199, 130],
    [137, 199, 131],
    [138, 45, 148],
    [138, 136, 29],
    [138, 199, 131],
    [138, 200, 131],
    [138, 200, 132],
    [138, 201, 132],
    [139, 45, 149],
    [139, 137, 29],
    [139, 138, 29],
    [139, 201, 132],
    [139, 202, 133],
    [140, 46, 150],
    [140, 46, 151],
    [140, 139, 29],
    [140, 202, 133],
    [140, 203, 133],
    [140, 203, 134],
    [141, 46, 151],
    [141, 140, 30],
    [141, 203, 134],
    [141, 204, 134],
    [141, 205, 134],
    [142, 46, 152],
    [142, 141, 30],
    [142, 205, 135],
    [142, 206, 135],
    [142, 206, 136],
    [143, 47, 153],
    [143, 47, 154],
    [143, 142, 30],
    [143, 207, 136],
    [143, 208, 136],
    [143, 208, 137],
    [144, 47, 155],
    [144, 143, 30],
    [144, 208, 137],
    [144, 209, 137],
    [144, 209, 138],
    [145, 47, 155],
    [145, 47, 156],
    [145, 144, 30],
    [145, 209, 138],
    [146, 48, 156],
    [146, 48, 157],
    [146, 145, 31],
    [147, 48, 158],
    [147, 146, 31],
    [148, 48, 158],
    [148, 48, 159],
    [148, 147, 31],
    [149, 49, 160],
    [149, 148, 31],
    [150, 49, 161],
    [150, 148, 31],
    [151, 49, 161],
    [151, 49, 162],
    [151, 49, 163],
    [151, 149, 32],
    [152, 50, 163],
    [152, 150, 32],
    [152, 151, 32],
    [153, 50, 164],
    [153, 152, 32],
    [154, 50, 165],
    [154, 50, 166],
    [154, 153, 32],
    [155, 51, 167],
    [155, 154, 33],
    [156, 51, 167],
    [156, 155, 33],
    [157, 51, 168],
    [157, 51, 169],
    [157, 156, 33],
    [158, 51, 169],
    [158, 52, 170],
    [158, 157, 33],
    [159, 52, 171],
    [159, 158, 33],
    [160, 52, 171],
    [160, 52, 172],
    [160, 159, 34],
    [161, 53, 173],
    [161, 159, 34],
    [162, 53, 174],
    [162, 160, 34],
    [163, 53, 175],
    [163, 53, 176],
    [163, 161, 34],
    [164, 54, 176],
    [164, 162, 34],
    [165, 54, 177],
    [165, 163, 35],
    [165, 164, 35],
    [166, 54, 178],
    [166, 54, 179],
    [166, 165, 35],
    [167, 54, 179],
    [167, 54, 180],
    [167, 55, 180],
    [167, 166, 35],
    [168, 55, 180],
    [168, 55, 181],
    [168, 167, 35],
    [169, 55, 181],
    [169, 55, 182],
    [169, 168, 36],
    [170, 55, 182],
    [170, 55, 183],
    [170, 56, 183],
    [170, 169, 36],
    [171, 56, 183],
    [171, 56, 184],
    [171, 170, 36],
    [172, 56, 184],
    [172, 56, 185],
    [172, 171, 36],
    [173, 56, 186],
    [173, 57, 186],
    [173, 171, 36],
    [174, 57, 186],
    [174, 57, 187],
    [174, 172, 37],
    [175, 57, 187],
    [175, 57, 188],
    [175, 173, 37],
    [176, 57, 189],
    [176, 174, 37],
    [177, 58, 190],
    [177, 58, 191],
    [177, 175, 37],
    [178, 58, 191],
    [178, 58, 192],
    [178, 176, 37],
    [179, 58, 192],
    [179, 177, 38],
    [179, 178, 38],
    [180, 59, 193],
    [180, 59, 194],
    [180, 179, 38],
    [181, 59, 194],
    [181, 59, 195],
    [181, 180, 38],
    [182, 59, 195],
    [182, 59, 196],
    [182, 60, 196],
    [182, 181, 38],
    [183, 60, 196],
    [183, 60, 197],
    [183, 182, 38],
    [184, 60, 197],
    [184, 60, 198],
    [184, 183, 39],
    [185, 60, 198],
    [185, 60, 199],
    [185, 183, 39],
    [186, 61, 199],
    [186, 61, 200],
    [186, 184, 39],
    [187, 61, 200],
    [187, 61, 201],
    [187, 185, 39],
    [188, 61, 202],
    [188, 186, 39],
    [189, 62, 202],
    [189, 62, 203],
    [189, 187, 40],
    [190, 62, 203],
    [190, 188, 40],
    [191, 189, 40],
    [192, 190, 40],
    [192, 191, 40],
    [193, 192, 41],
    [194, 193, 41],
    [195, 194, 41],
    [196, 195, 41],
    [197, 195, 41],
    [198, 196, 42],
    [199, 197, 42],
    [200, 198, 42],
    [201, 199, 42],
    [202, 200, 42],
    [203, 201, 43],
    [204, 202, 43],
    [205, 203, 43],
    [205, 204, 43],
    [206, 205, 43],
    [207, 206, 44],
    [208, 207, 44],
    [209, 207, 44],
    [209, 208, 44],
    [210, 208, 44],
    [211, 209, 44],
    [212, 210, 45],
    [213, 211, 45],
    [214, 212, 45],
    [215, 213, 45],
    [216, 214, 45],
    [217, 215, 46],
    [218, 216, 46],
    [218, 217, 46],
    [219, 217, 46],
    [219, 218, 46],
    [220, 218, 46],
    [220, 219, 46],
    [221, 219, 46],
    [221, 220, 46],
    [221, 220, 47],
    [222, 220, 47],
    [223, 221, 47],
    [223, 222, 47],
    [224, 222, 47],
    [224, 223, 47],
    [225, 223, 47],
    [226, 224, 47],
    [226, 224, 48],
    [227, 225, 48],
    [228, 226, 48],
    [229, 227, 48],
    [230, 228, 48],
    [231, 229, 48],
    [231, 229, 49],
    [231, 230, 49],
    [232, 230, 49],
    [232, 231, 49],
    [233, 231, 49],
    [233, 232, 49],
    [234, 232, 49],
    [234, 233, 49],
    [235, 233, 49],
    [236, 234, 50],
    [237, 235, 50],
]


@register_dsetcreator(dset_name=dset_name)
def create_dset():
    dset_creator = DSetCreator(dset_name=dset_name)
    dset_creator.download(
        url="http://ftp.itec.aau.at/datasets//GLENDA/downloads//Glenda_v1.5_classes.zip",
        file_name="Glenda_v1.5_classes.zip",
        data_kind=DataKind.TRAINING_DATA,
    )
    dset_path = dset_creator.unpack_and_store()
    masks = list((dset_path / DataKind.TRAINING_DATA / "Glenda_v1.5_classes" / "annots").rglob("*.png"))
    keys = list(class_mapping.keys())
    dset_creator.transform_masks(
        masks=masks,
        load="rgb",
        transform={pix_val: idx for idx, pix_val in enumerate(keys)},
        train=True,
        ignore=[tuple(vals) for vals in wrongs],
    )
    return dset_path


@register_taskcreator(task_name="glenda_endometriosis_segmentation", dset_name=dset_name)
def create_task_glenda(dset_path: Path):
    task = TaskCreator(
        dset_path=dset_path,
        name="glenda_endometriosis_segmentation",
        task_type=TaskType.SEMANTIC_SEGMENTATION,
        desc="The dataset includes region-based annotations of 4 pathological endometriosis categories"
        " as well as non pathological counter example images",
        ref=REFERENCE,
        url="http://ftp.itec.aau.at/datasets//GLENDA",
        instr="download data Glenda_v1.5_classes and GLENDA_v1.5_no_pathology via website",
        lic=License.CC_BY_NC_4_0,
        release="2020",
        keywords=[Keyword.LAPAROSCOPY, Keyword.MEDICAL, Keyword.GYNECOLOGY],
    )
    data_iterator = []
    for img_id in [
        p.stem
        for p in (dset_path / DataKind.TRAINING_DATA / "Glenda_v1.5_classes" / "frames").iterdir()
        if p.is_file() and p.suffix == ".jpg"
    ]:
        data_iterator.append(
            {
                Modality.SAMPLE_ID: img_id,
                Modality.IMAGE: dset_path / DataKind.TRAINING_DATA / "Glenda_v1.5_classes" / "frames" / f"{img_id}.jpg",
                Modality.MASK: (
                    dset_path
                    / DataKind.TRAINING_LABELS
                    / "transformed_masks"
                    / "Glenda_v1.5_classes"
                    / "annots"
                    / f"{img_id}.png"
                ),
            }
        )
    task.find_data(
        train_iterator=data_iterator, idx_to_class={ix: cls for ix, cls in enumerate(class_mapping.values())}
    )
    task.split_folds(n_folds=5)
    task.infer_stats()
    task.push_and_test()
